/////////////////////////////////////////////
// 902P Get SP - [Consigne Cible] Consigne vise
QString TPG300::getsp(void)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return valeur_retour;        

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];
  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x53'; // C1 = mnemonique du parametre      SP
  tableau_instruction[6] = '\x50'; // C2 = mnemonique du parametre

  tableau_instruction[7] = '\x05'; // <ENQ> Attente de reponse

  tableau_instruction[8] = '\0'; // <NUL> on ferme le tableau

  send_rs232_char(tableau_instruction , 8);

  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 10);

/* Explications de la reception 
octet 1 = <STX> = 02
octets 2 et 3 = SP
octets 4  9 = valeur numrique
octet 10 = <ETX> = 03
octet 11 = BCC = caractere de verification = OU exclusif entre tous les caracteres compris entre l'octet 2 et 10 (inclus)
*/

  if(tableau_reception[0] != '\x02' || tableau_reception[1] != '\x53' ||  tableau_reception[2] != '\x50' ||  tableau_reception[9] != '\x03') 
  {
    valeur_retour = "Error";
    return valeur_retour;
  }

  // si le dernier caractere de la valeur est un point, on le supprime
  if(tableau_reception[8] == '\x2E') 
    tableau_reception[8] = '\x20'; 

  // on recupere la valeur des octets de donnees et on le convertit en chaine de caractres sur 6 espaces
  QChar c4 = (unsigned char) tableau_reception[3];
  QChar c5 = (unsigned char) tableau_reception[4];
  QChar c6 = (unsigned char) tableau_reception[5];
  QChar c7 = (unsigned char) tableau_reception[6];
  QChar c8 = (unsigned char) tableau_reception[7];
  QChar c9 = (unsigned char) tableau_reception[8];

  QString consigne = "";
  consigne = consigne + c4 + c5 + c6 + c7 + c8 + c9;

  consigne = consigne.replace("\x20", "");
  
  return consigne;

}



/////////////////////////////////////////////
// 902P Get OP - [Puissance de sortie]
QString TPG300::getop(void)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return valeur_retour;        

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];
  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x4F'; // C1 = mnemonique du parametre      OP
  tableau_instruction[6] = '\x50'; // C2 = mnemonique du parametre

  tableau_instruction[7] = '\x05'; // <ENQ> Attente de reponse

  tableau_instruction[8] = '\0'; // <NUL> on ferme le tableau

  send_rs232_char(tableau_instruction , 8);

  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 10);

/* Explications de la reception 
octet 1 = <STX> = 02
octets 2 et 3 = OP
octets 4  9 = valeur numrique
octet 10 = <ETX> = 03
octet 11 = BCC = caractere de verification = OU exclusif entre tous les caracteres compris entre l'octet 2 et 10 (inclus)
*/

  if(tableau_reception[0] != '\x02' || tableau_reception[1] != '\x4F' ||  tableau_reception[2] != '\x50' ||  tableau_reception[9] != '\x03') 
  {
    valeur_retour = "Error";
    return valeur_retour;
  }

  // si le dernier caractere de la valeur est un point, on le supprime
  if(tableau_reception[8] == '\x2E') 
    tableau_reception[8] = '\x20'; 

  // on recupere la valeur des octets de donnees et on le convertit en chaine de caractres sur 6 espaces
  QChar c4 = (unsigned char) tableau_reception[3];
  QChar c5 = (unsigned char) tableau_reception[4];
  QChar c6 = (unsigned char) tableau_reception[5];
  QChar c7 = (unsigned char) tableau_reception[6];
  QChar c8 = (unsigned char) tableau_reception[7];
  QChar c9 = (unsigned char) tableau_reception[8];

  QString puissance = "";
  puissance = puissance + c4 + c5 + c6 + c7 + c8 + c9;

  puissance = puissance.replace("\x20", "");
  
  return puissance;

}


/////////////////////////////////////////////
// 902P Get PV - [Lecture temperature]
QString TPG300::getpv(void)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return valeur_retour;        

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];
  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x50'; // C1 = mnemonique du parametre      PV
  tableau_instruction[6] = '\x56'; // C2 = mnemonique du parametre

  tableau_instruction[7] = '\x05'; // <ENQ> Attente de reponse

  tableau_instruction[8] = '\0'; // <NUL> on ferme le tableau

  send_rs232_char(tableau_instruction , 8);

  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 10);

/* Explications de la reception 
octet 1 = <STX> = 02
octets 2 et 3 = PV
octets 4  9 = valeur numrique
octet 10 = <ETX> = 03
octet 11 = BCC = caractere de verification = OU exclusif entre tous les caracteres compris entre l'octet 2 et 10 (inclus)
*/

  if(tableau_reception[0] != '\x02' || tableau_reception[1] != '\x50' ||  tableau_reception[2] != '\x56' ||  tableau_reception[9] != '\x03') 
  {
    valeur_retour = "Error";
    return valeur_retour;
  }

  // si le dernier caractere de la valeur est un point, on le supprime
  if(tableau_reception[8] == '\x2E') 
    tableau_reception[8] = '\x20'; 

  // on recupere la valeur des octets de donnees et on le convertit en chaine de caractres sur 6 espaces
  QChar c4 = (unsigned char) tableau_reception[3];
  QChar c5 = (unsigned char) tableau_reception[4];
  QChar c6 = (unsigned char) tableau_reception[5];
  QChar c7 = (unsigned char) tableau_reception[6];
  QChar c8 = (unsigned char) tableau_reception[7];
  QChar c9 = (unsigned char) tableau_reception[8];

  QString temperature = "";
  temperature = temperature + c4 + c5 + c6 + c7 + c8 + c9;

  temperature = temperature.replace("\x20", "");
  
  return temperature;

}


/////////////////////////////////////////////
// 902P Get mode - [manuel ou automatique]
QString TPG300::getmode(void)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return valeur_retour;        

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];
  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x53'; // C1 = mnemonique du parametre      SW
  tableau_instruction[6] = '\x57'; // C2 = mnemonique du parametre

  tableau_instruction[7] = '\x05'; // <ENQ> Attente de reponse

  tableau_instruction[8] = '\0'; // <NUL> on ferme le tableau

  send_rs232_char(tableau_instruction , 8);

  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 10);

/* Explications de la reception 
octet 1 = <STX> = 02
octets 2 et 3 = SW
octets 4  9 = valeur numrique
octet 10 = <ETX> = 03
octet 11 = BCC = caractere de verification = OU exclusif entre tous les caracteres compris entre l'octet 2 et 10 (inclus)
*/

  if(tableau_reception[0] != '\x02' || tableau_reception[1] != '\x53' ||  tableau_reception[2] != '\x57' ||  tableau_reception[3] != '\x3E' ||  tableau_reception[8] != '\x03') 
  {
    valeur_retour = "Error";
    return valeur_retour;
  }

  // on recupere la valeur des octets de donnees et on le convertit en chaine de caractres sur 6 espaces
  QChar c5 = (unsigned char) tableau_reception[4];
  QChar c6 = (unsigned char) tableau_reception[5];
  QChar c7 = (unsigned char) tableau_reception[6];
  QChar c8 = (unsigned char) tableau_reception[7];

  QString mode_binaire = "";
  mode_binaire = mode_binaire + c5;
  int mode_binaire_int = mode_binaire.toInt();

  QString mode = "";
  if(mode_binaire_int >=8)
    mode = "Manuel";
  else
    mode = "Automatique";

  return mode;

}


/////////////////////////////////////////////
// 902P Set Consigne - [Consigne Cible] Consigne vise
void TPG300::setsp(double val)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return;        

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];

  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x02'; // <STX>

  tableau_instruction[6] = '\x53'; // C1 = mnemonique du parametre      SL
  tableau_instruction[7] = '\x4C'; // C2 = mnemonique du parametre
// SL = '53'+'4C' consigne 1 ==> marche pas
// S2 = "53"+"32" consigne 2 ==> marche pas
// RT = '52'+'54' consigne locale ==> marche pas <nak>
// RI = '52'+'49' consigne externe ==> marche pas <nak>

  tableau_instruction[8] = '\0'; // <NUL> on ferme le tableau

  tableau_instruction[9] = '\x03'; // <ETX> 

  // on transforme le tableau en un QString
  QChar c0 = (unsigned char) tableau_instruction[0];
  QChar c1 = (unsigned char) tableau_instruction[1];
  QChar c2 = (unsigned char) tableau_instruction[2];
  QChar c3 = (unsigned char) tableau_instruction[3];
  QChar c4 = (unsigned char) tableau_instruction[4];
  QChar c5 = (unsigned char) tableau_instruction[5];
  QChar c6 = (unsigned char) tableau_instruction[6];
  QChar c7 = (unsigned char) tableau_instruction[7];
  QChar c9 = (unsigned char) tableau_instruction[9];

  QString instruction = "";
  instruction = instruction + c0 + c1 + c2 + c3 + c4 + c5 + c6 + c7;
  instruction = instruction + QString::number(val);
  instruction = instruction + c9;

  // creation du BCC 
  QString chaine_calcul_bbc = "";
  chaine_calcul_bbc = chaine_calcul_bbc +c6 + c7 + QString::number(val) + c9;
  QByteArray chaine_calcul_bbc_byte = chaine_calcul_bbc.toLatin1();
  QString bcc_string = bcc((unsigned char*)chaine_calcul_bbc_byte.data() , chaine_calcul_bbc_byte.size());

  instruction = instruction + bcc_string;

  QByteArray instruction_byte;
  instruction_byte = instruction.toLatin1();

  send_rs232_char( instruction_byte.data() , instruction.size() );

  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 1);

/* Explications de la reception 
octet 1 = <ACK> ou <NACK>
*/

return;
}


/////////////////////////////////////////////
// 902P Set puissance sortie
void TPG300::setop(double val)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return;        

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];

  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x02'; // <STX>

  tableau_instruction[6] = '\x4F'; // C1 = mnemonique du parametre      OP
  tableau_instruction[7] = '\x50'; // C2 = mnemonique du parametre

  tableau_instruction[8] = '\0'; // <NUL> on ferme le tableau

  tableau_instruction[9] = '\x03'; // <ETX> 

  // on transforme le tableau en un QString
  QChar c0 = (unsigned char) tableau_instruction[0];
  QChar c1 = (unsigned char) tableau_instruction[1];
  QChar c2 = (unsigned char) tableau_instruction[2];
  QChar c3 = (unsigned char) tableau_instruction[3];
  QChar c4 = (unsigned char) tableau_instruction[4];
  QChar c5 = (unsigned char) tableau_instruction[5];
  QChar c6 = (unsigned char) tableau_instruction[6];
  QChar c7 = (unsigned char) tableau_instruction[7];
  QChar c9 = (unsigned char) tableau_instruction[9];

  QString instruction = "";
  instruction = instruction + c0 + c1 + c2 + c3 + c4 + c5 + c6 + c7;
  instruction = instruction + QString::number(val);
  instruction = instruction + c9;

  // creation du BCC 
  QString chaine_calcul_bbc = "";
  chaine_calcul_bbc = chaine_calcul_bbc +c6 + c7 + QString::number(val) + c9;
  QByteArray chaine_calcul_bbc_byte = chaine_calcul_bbc.toLatin1();
  QString bcc_string = bcc((unsigned char*)chaine_calcul_bbc_byte.data() , chaine_calcul_bbc_byte.size());

  instruction = instruction + bcc_string;

  QByteArray instruction_byte;
  instruction_byte = instruction.toLatin1();

  send_rs232_char( instruction_byte.data() , instruction.size() );


  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 1);

/* Explications de la reception 
octet 1 = <ACK> ou <NACK>
*/

return;
}



/////////////////////////////////////////////
// 902P Set mode - [manuel ou automatique]
void TPG300::setmode(int val)
{
  QString valeur_retour = "-1"; // par defaut, on met a -1

  if(port == NULL) // Il y a un probleme
    return;        

  if(val != 1 && val != 2)
    return;

  // on vide tout ce qu'il pourrait y avoir a recevoir
  receive_rs232_rapide();

  // creation d'un tableau
  char tableau_instruction[255];

  tableau_instruction[0] = '\x04'; // <EOT> (End Of Text)

  tableau_instruction[1] = '\x30'; // AG = 00 (numero d'adresse de groupe du regulateur), groude sd
  tableau_instruction[2] = '\x30'; 

  tableau_instruction[3] = '\x31'; // AU = 11 (numero d'adresse d'unite du regulateur), unity ident
  tableau_instruction[4] = '\x31'; 

  tableau_instruction[5] = '\x02'; // <STX>

  tableau_instruction[6] = '\x53'; // C1 = mnemonique du parametre      SW
  tableau_instruction[7] = '\x57'; // C2 = mnemonique du parametre

  tableau_instruction[8] = '\x3E'; //  >

  if( val == 2) // MANUEL = "8"
    tableau_instruction[9] = '\x38'; //  8  
  else if ( val == 1) // AUTOMATIQUE = "0"
    tableau_instruction[9] = '\x30'; //  0  

  tableau_instruction[10] = '\x30'; //  0
  tableau_instruction[11] = '\x32'; //  2
  tableau_instruction[12] = '\x30'; //  0

  tableau_instruction[13] = '\x03'; // <ETX> 

  tableau_instruction[14] = '\0'; // <NUL> on ferme le tableau


  // on transforme le tableau en un QString
  QChar c0 = (unsigned char) tableau_instruction[0];
  QChar c1 = (unsigned char) tableau_instruction[1];
  QChar c2 = (unsigned char) tableau_instruction[2];
  QChar c3 = (unsigned char) tableau_instruction[3];
  QChar c4 = (unsigned char) tableau_instruction[4];
  QChar c5 = (unsigned char) tableau_instruction[5];
  QChar c6 = (unsigned char) tableau_instruction[6];
  QChar c7 = (unsigned char) tableau_instruction[7];
  QChar c8 = (unsigned char) tableau_instruction[8];
  QChar c9 = (unsigned char) tableau_instruction[9];
  QChar c10 = (unsigned char) tableau_instruction[10];
  QChar c11= (unsigned char) tableau_instruction[11];
  QChar c12 = (unsigned char) tableau_instruction[12];
  QChar c13 = (unsigned char) tableau_instruction[13];

  QString instruction = "";
  instruction = instruction + c0 + c1 + c2 + c3 + c4 + c5 + c6 + c7 + c8 + c9 + c10 + c11 + c12 + c13;

  // creation du BCC 
  QString chaine_calcul_bbc = "";
  chaine_calcul_bbc = chaine_calcul_bbc +c6 + c7 + c8 + c9 + c10 + c11 + c12 + c13;
  QByteArray chaine_calcul_bbc_byte = chaine_calcul_bbc.toLatin1();
  QString bcc_string = bcc((unsigned char*)chaine_calcul_bbc_byte.data() , chaine_calcul_bbc_byte.size());

  instruction = instruction + bcc_string;

  QByteArray instruction_byte;
  instruction_byte = instruction.toLatin1();

  send_rs232_char( instruction_byte.data() , instruction.size() );

  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 1);

/* Explications de la reception 
octet 1 = <ACK> ou <NACK>
*/

return;



}










unsigned int TPG300::Crc16(unsigned char *Adresse_tab , unsigned char Taille_max)
{
  unsigned int Crc = 0xFFFF;
  unsigned int Polynome = 0xA001;
  unsigned char CptOctet = 0;
  unsigned char CptBit = 0;
  unsigned char Parity= 0;

  Crc = 0xFFFF;
  Polynome = 0xA001; // Polynme = 2^15 + 2^13 + 2^0 = 0xA001.

  for ( CptOctet= 0 ; CptOctet < Taille_max ; CptOctet++)
  {
    Crc ^= *( Adresse_tab + CptOctet); //Ou exculsif entre octet message et CRC

    for ( CptBit = 0; CptBit <= 7 ; CptBit++) // Mise a 0 du compteur nombre de bits 
    {
      Parity= Crc;
      Crc >>= 1; // Dcalage a droite du crc
      if (Parity%2 == TRUE) Crc ^= Polynome; // Test si nombre impair -> Apres decalage  droite il y aura une retenue
    } // "ou exclusif" entre le CRC et le polynome generateur.
  }


  // on recupere la valeur du Crc et on le convertit en chaine de caractres sur 4 espaces
  QString chaine = QString::number(Crc, 16); // Base = 16 for hex
  if(chaine.length () == 0)
    chaine = "0000";
  else if(chaine.length () == 1)
    chaine = "000" + chaine;
  else if(chaine.length () == 2)
    chaine = "00" + chaine;
  else if(chaine.length () == 3)
    chaine = "0" + chaine;

  QString chainePf = chaine;
  chainePf = QString::QString(chainePf.at(2)) + QString::QString(chainePf.at(3));

  QString chainePF = chaine;
  chainePF = QString::QString(chainePF.at(0)) + QString::QString(chainePF.at(1));

  bool ok;
  int Pf_int = chainePf.toInt(&ok, 16); // Base = 16 for hexadecimal
  int PF_int = chainePF.toInt(&ok, 16); // Base = 16 for hexadecimal

  Adresse_tab[Taille_max] = Pf_int;
  Adresse_tab[Taille_max+1] = PF_int;

  return(Crc);
}


/////////////////////////////////////////////
// Realisation de l'octet de controle
// OU EXCLUSIF
QString TPG300::bcc(unsigned char *adresse_tab , int taille)
{
  unsigned char bcc = '\0';
  
  int i = 0;
  for(i=0 ; i < taille ; i++)
  {
    bcc = bcc ^ adresse_tab[i];
  }

  QChar bcc_char = bcc;
  QString bcc_string = "";
  bcc_string = bcc_string + bcc_char;

  return(bcc_string);
}

