#include "tpg300.h"

#include "send_receive.cpp" // Fonctions pour envoyer et recevoir
#include "fonctions_dialogues.cpp" // Fonctions de dialogue avec le TPG300

TPG300::TPG300(QWidget *parent) : QMainWindow(parent)
{
  ui.setupUi(this); // chargement de l'interface graphique

  port = NULL; // initialisation du port serie

  // creation du thread qui met a jour les valeurs
  thread.pt = this; // on donne le pointeur vers TPG300
  thread.start();

}

/////////////////////////////////////////////
// Configuration de la connexion RS232
/////////////////////////////////////////////
void TPG300::on_pushButton_1_1_clicked()
{
  // on affiche la fenetre
  fenetre_rs232.show();
}

/////////////////////////////////////////////
// Configuration de PARAMETRES
/////////////////////////////////////////////
void TPG300::on_pushButton_1_2_clicked()
{
  // on affiche la fenetre
  fenetre_parametres.show();
}

/////////////////////////////////////////////
// Configuration de consigne
/////////////////////////////////////////////
void TPG300::on_pushButton_clicked()
{
  // on affiche la fenetre
  fenetre_tsp.show();
}

/////////////////////////////////////////////
// Configuration de Puissance
/////////////////////////////////////////////
void TPG300::on_pushButton_2_clicked()
{
  // on affiche la fenetre
  fenetre_top.show();
}

/////////////////////////////////////////////
// Configuration de mode
/////////////////////////////////////////////
void TPG300::on_pushButton_3_clicked()
{
  // on affiche la fenetre
  fenetre_ma.show();




}


/////////////////////////////////////////////
// Connexion a l'appareil
/////////////////////////////////////////////
void TPG300::on_pushButton_1_3_clicked()
{

  // on efface le moniteur
  ui.textEdit_2_1->clear();
  ui.textEdit_1_1->clear(); // on efface les infos sur les fichiers d'enregistrement


  // affiche sur le Moniteur que l'on a appuye sur le bouton connexion
  affichage_moniteur("Ouverture du port de communication");

  int erreur;
  // on configure le port de communication
  erreur = fenetre_rs232.connexion(port); // permet de se connecter a l'appareil

  if(erreur == -1)
  {
    affichage_moniteur("==> Erreur de connexion");
    thread.etat = 0;
    return;
  }
  else
  {
    port = (QextSerialPort *) erreur;
  }

  // on vide les ventuelles choses en attente sur la ligne
  char tableau_reception[255];
  receive_rs232_char(tableau_reception , 240);


  ui.lineEdit_1_8->setText("0"); // nombre de cycles de mesures
  nb_cycles = 0; // initialisation a 0

  QDateTime date = QDateTime::currentDateTime();
  date_init = date;

  ui.lineEdit_1_5->setText(date.toString( "dd/MM/yyyy") ); // Date de dpart
  ui.lineEdit_1_6->setText(date.toString( "HH:mm:ss") ); // Heure de dpart
  nom_file = "data/data_902P_" + date.toString( "yyyy-MM-dd_HH-mm-ss") + ".txt";
  ui.textEdit_1_1->insertPlainText( "./" + nom_file + "\n" ); // Nom du fichier d'enregistrement
  ui.textEdit_1_1->insertPlainText( "./data/data_902P_current.txt" ); // Nom du fichier d'enregistrement
  ui.lineEdit_1_7->setText( date.toString( "dd/MM/yyyy - HH:mm:ss") );

  // On complte le fichier de mesures
  QFile file;
  file.setFileName("data/data_902P_" + date.toString( "yyyy-MM-dd_HH-mm-ss") + ".txt");
  file.open(QIODevice::Append | QIODevice::Text );
  QTextStream out(&file);

  affichage_moniteur("Date et heure de dpart ==> " + date.toString( "dd/MM/yyyy - HH:mm:ss") );
  out << "Date et heure de dpart : " << date.toString( "dd/MM/yyyy - HH:mm:ss") << endl;


  QString get_pv = getpv();
  affichage_moniteur("Temprature ==> " + get_pv);
  out << "Temprature : " << get_pv << endl;
  ui.lineEdit_1_1->setText(get_pv ); 

  QString get_tsp = getsp();
  affichage_moniteur("Consigne vise ==> " + get_tsp);
  out << "Consigne vise : " << get_tsp << endl;
  ui.lineEdit_1_3->setText(get_tsp ); 

  QString get_top = getop();
  affichage_moniteur("Puissance de Sortie cible souhaite ==> " + get_top);
  out << "Puissance de Sortie cible souhaite : " << get_top << endl;
  ui.lineEdit_1_2->setText(get_top ); 

  QString get_mA = getmode();
  affichage_moniteur("Mode de fonctionnement ==> " + get_mA);
  out << "Mode de fonctionnement : " << get_mA << endl;
  ui.lineEdit_1_4->setText(get_mA ); 

  out << endl;
  out << endl;

  out << "Date et heure de la mesure" << "\x09" << "Temps coul (s)" << "\x09" << "Temprature (C)" << "\x09" << "Consigne (C)" << "\x09" << "Puissance de Sortie (%)"  << "\x09" << "Mode"  << endl;
  out << date.toString( "dd/MM/yyyy HH:mm:ss") << "\x09" << "0" << "\x09" << get_pv << "\x09" << get_tsp << "\x09" << get_top << "\x09" << get_mA << endl;
  
  file.close();


  // on enregistre les donnees dans le fichier courant
  // On complte le fichier de mesures
  QFile file_current;
  file.setFileName("data/data_902P_current.txt");
  if( file.exists() )
    file.remove();
  file.open(QIODevice::Append | QIODevice::Text );
  QTextStream out_current(&file);

  out_current << "Date et heure de la mesure" << "\x09" << ":" << "\x09" << date.toString( "dd/MM/yyyy HH:mm:ss") << endl;
  out_current << "PV [Temperature]" << "\x09" << ":" << "\x09" << get_pv << endl;
  out_current << "SP [Consigne]" << "\x09" << ":" << "\x09" << "" << endl;
  out_current << "OP [Puissance de Sortie]" << "\x09" << ":" << "\x09" << get_top << endl;
  out_current << "m-A [Mode Manuel]" << "\x09" << ":" << "\x09" << get_mA;  

  file_current.close();

  thread.etat = 1; // demarrage des mesures du thread

}



/////////////////////////////////////////////
// Affichage sur le moniteur
void TPG300::affichage_moniteur(QString la_commande)
{
  ui.textEdit_2_1->insertPlainText( la_commande.toLatin1() + "\n" ); 
}


/////////////////////////////////////////////
// Mesure des valeurs de l'appareil 2704
/////////////////////////////////////////////
void TPG300::mesure(void)
{

  nb_cycles++; // ajout d'un cycle
  ui.lineEdit_1_8->setText(QString::number(nb_cycles)); // nombre de cycles de mesures

  QDateTime date = QDateTime::currentDateTime();
  int temps_ecoule = date_init.secsTo(date);

   // on regarde s'il faut modifier quelque chose
  if(fenetre_ma.mode != 0)
  {
     setmode(fenetre_ma.mode);
     fenetre_ma.mode = 0;
  }
  if(fenetre_tsp.change != 0)
  {
     fenetre_tsp.change = 0;
     setsp(fenetre_tsp.consigne);
  }
  if(fenetre_top.change != 0)
  {
    fenetre_top.change = 0;
    setop(fenetre_top.puissance);
  }

  // on fait les mesures de pression

  QString get_pv = getpv();
  QString get_tSP = getsp();
  QString get_T_OP = getop();
  QString get_mA = getmode();

  // on affiche les resultats
  ui.lineEdit_1_1->setText(get_pv ); 
  ui.lineEdit_1_3->setText(get_tSP ); 
  ui.lineEdit_1_2->setText(get_T_OP ); 
  ui.lineEdit_1_4->setText(get_mA); 

  // on affiche l'heure
  ui.lineEdit_1_7->setText( date.toString( "dd/MM/yyyy - HH:mm:ss") );

  // On complte le fichier de mesures
  QFile file;
  file.setFileName(nom_file);
  file.open(QIODevice::Append | QIODevice::Text );
  QTextStream out(&file);
  out << date.toString( "dd/MM/yyyy HH:mm:ss") << "\x09" << temps_ecoule << "\x09" << get_pv << "\x09" << get_tSP << "\x09" << get_T_OP << "\x09" << get_mA << endl;
  file.close();

  // on enregistre les donnees dans le fichier courant
  QFile file_current;
  file_current.setFileName("data/data_902P_current.txt");
  if( file_current.exists() )
    file_current.remove();
  file_current.open(QIODevice::Append | QIODevice::Text );
  QTextStream out_current(&file_current);
  out_current << "Date et heure de la mesure" << "\x09" << ":" << "\x09" << date.toString( "dd/MM/yyyy HH:mm:ss") << endl;
  out_current << "PV [Temperature]" << "\x09" << ":" << "\x09" << get_pv << endl;
  out_current << "SP [Consigne]" << "\x09" << ":" << "\x09" << get_tSP << endl;
  out_current << "OP [Puissance de Sortie]" << "\x09" << ":" << "\x09" << get_T_OP << endl;
  out_current << "m-A [Mode Manuel]" << "\x09" << ":" << "\x09" << get_mA;  
  file_current.close();

}


